package interfacesFerrovia;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Interface para acesso aos comandos da rede ferroviria.
 * @author Fernando Vanini
 *
 */

public interface ControleFerrovia extends DefsGlobais{
	
	/**
	 * Registra um tratador para os eventos gerados pela rede ferroviria.
	 * Pode-se registrar mais de um tratador
	 * @param tratador Objeto que implementa a interface TratadorDeEventos.
	 * @return true se o registro for feito normalmente.
	 */
	public boolean registraTratadorDeEventos(TratadorDeEventos tratador);
	
	/**
	 * Consulta aos elementos da rede. Retorna uma lista contendo os id's dos
	 * elementos da rede.
	 * @return lista contendo os id's dos elementos da rede.
	 */
	public String[] consultaElementos();
	
	/**
	 * Consulta os atributos fixos de um elemento da rede.
	 * @param idElemento identificao nica do elemento.
	 * @return mapa <atributo,valor> contendo os atributos fixos do elemento.
	 */
	public Map<TipoAtributo,Object> consultaAtributosFixos(String idElemento);
	
	/**
	 * Consulta o estado (formado pelo conjunto de atributos variveis) de
	 * um elemento da rede.
	 * @param idElemento identificao nica do elemento.
	 * @return mapa <atributo,valor> que caracteriza o estado do elemento.
	 */
	public Map<TipoAtributo,Object> consultaEstado(String idElemento);
	
	/**
	 * Consulta um atributo de um elemento da rede.
	 * @param idElemento Identificao nica do elemento
	 * @param atributo   Nome do atributo a ser consultado
	 * @return Object    Valor do atributo
	 */
	public Object consultaAtributo(String idElemento, TipoAtributo atributo);
	
	/**
	 * Consulta o conjunto de atributos de um dado elemento da rede.
	 * @param idElemento Identificao nica do elemento da rede.
	 * @return Conjunto de atributos (fixos e variveis) associados ao elemento.
	 */
	public Set<TipoAtributo> consultaListaDeAtributos(String idElemento);
	
	/**
	 * Comando para acelerar uma composico.
	 * @param idComposicao Identificao nica da composio.
	 * @param velocidade   Velocidade a ser atingida.
	 * @param urgente      Urgente(true) ou normal(false).
	 * @return             Tempo necessrio para atingir a velocidade.
	 */
	public int acelerarComposicao(String idComposicao, float velocidade, boolean urgente);
	
	/**
	 * Comando para acelerar a composicao
	 * @param idComposicao  Identificao nica da composio
	 * @param velocidade    Velocidade a ser atingida
	 * @param acel          Taxa de acelerao 
	 * @return              Tempo necessrio para atingir a velocidade
	 */
	public int acelerarComposicao(String idComposicao, float velocidade, float acel);
	
	/**
	 * Comando para frear a composio.
	 * @param idComposicao Identificao nica da composio.
	 * @param velocidade   Velocidade a ser atingida
	 * @param urgente      Urgente(true) ou normal(false).
	 * @return             Tempo necessrio para realizar a operao.
	 */
	public int frearComposicao(String idComposicao, float velocidade, boolean urgente);
	
	/**
	 * Comando para frear a composio
	 * @param idComposicao Identificao nica da composio.
	 * @param velocidade   Velocidade a ser atingida.
	 * @param fren         Taxa de frenagem aplicada.
	 * @return             Tempo necessrio para realizar a operao.
	 */
	public int frearComposicao(String idComposicao, float velocidade, float fren);
	
	/**
	 * COnsulta a posio prevista para um instante futuro.
	 * @param idComposicao Identificao nica da composio.
	 * @param instante Instante futuro a ser considerado.
	 * @return A posio prevista, no trecho atual.
	 */
	public float consultaPosicao(String idComposicao, int instante);
	
	/**
	 * Consulta o tempo previsto para atingir uma dada posio no trecho atual
	 * (considerando a velocidade e taxa de acelerao ou frenagem atuais).
	 * @param idComposicao Identificao nica da composio.
	 * @param posicao Posio a ser atingida.
	 * @return Tempo estimado necessrio para atingir a posio.
	 */
	public int consultaTempo(String idComposicao, float posicao);
	
	/**
	 * Calcula a distncia mnima necessria  parada de uma composio.
	 * @param idComposicao Identificao nica da composio.
	 * @return Distncia mnima necessria  parada da composio.
	 */
	public float consultaDistMinParada(String idComposicao);

	@SuppressWarnings("unchecked")
	public List consultaCoordenadas(String idTrecho);
	
	/**
	 * Comando para mudar o estado de uma chave.
	 * @param idChave    Identificao nica da chave
	 * @param idTrechoA  Identificao do trecho a ser conectado  extremidade A da chave.
	 * @param idTrechoB  Identificao do trecho a ser conectado  extremidade B da chave.
	 * @return           True se a operao pode ser realizada.
	 */
	public boolean mudarChave(String idChave, String idTrechoA, String idTrechoB);
	
	/**
	 * Define a resoluo do relgio do simulador (intervalo entre tics).
	 * (default: 15s, habilitado = false)
	 * @param intervalo  Intervalo entre cada tic
	 * @param habilitado Indica se o simulador gera ou no o evento relativo ao tic.
	 */
	//public void defineTic(int intervalo, boolean habilitado);	 eliminado => ser sempre 1 !!!!

	/**
	 * Define o cenrio a ser usado na simulao.
	 * @param cenario Indica o cenrio a ser usado (0..nCenarios-1)
	 */	
	public void defineCenario(int cenario);
	
	/**
	 *    Simula um cenrio pr-definido, formado por um conjunto de elementos de rede.
	 *    @param t1 Instante de incio da simulao (quase sempre zero).
	 *    @param t2 Instante de trmino da simulao.
	 * @throws Exception 
	 */
	public void simularCenario(int t1, int t2) throws Exception;
	
	

}
